#! /bin/perl 
use strict;
use warnings;
use autodie;
use Getopt::Long;
use vars qw ($help $gff $outbed $what);

$outbed="annotation_DIEGO.bed";
$what="gene";

GetOptions (
    "g=s"       => \$gff,
    "h"         => \$help,
    "w=s"       => \$what,
    "o=s"       => \$outbed
    );
usage() if ($help || !$gff);

open(my $IN, "<", $gff); 
my @files;
my @names;
my $count=1;
open(my $OUT, ">", $outbed);

while(<$IN>) {
    next if (/^#/);
    my @F=split;
    next unless ($F[2] eq $what);
    my $geneid="ID_$count";
    my $type="n.a.";
    my $genename="Gene_$count";
    $count++;
    if(/gene_id \"(\S+)\"/){
	$geneid=$1;
    }
    elsif(/gene_id=([A-Za-z0-9.]+)/) {
	$geneid=$1;
    }
    if(/gene_name \"(\S+)\"/){
	$genename=$1;
    }
    elsif(/gene_name=([A-Za-z0-9._]+)/){
	$genename=$1;
    }
    if(/gene_type \"(\S+)\"/){
	$type=$1;
    }
    elsif(/gene_type=([A-Za-z0-9._]+)/){
	$type=$1;
    }
    elsif(/gene_biotype \"(\S+)\"/){
	$type=$1;
    }
    print $OUT "$F[0]\t$F[3]\t$F[4]\t$type".";$geneid".";$genename\t0\t$F[6]\n";
  
}
close($OUT);
close($IN);
printf STDERR "You now should hav a file $outbed to play with\nThank you for travelling with us, Good bye!\n";

    
sub usage {
  print STDERR "\ngfftoDIEGObed.pl\n";
  print STDERR "usage: gfftoDIEGObed.pl -g <file> -o <file>\n";
  print STDERR "\n";
  print STDERR "[INPUT]\n";
  print STDERR " -g <file>    gff file e.g. from ENSMBL\n";
  print STDERR " -o <file>    output file name (default:annotation_DIEGO.bed)\n";
  print STDERR " -w name    what is used as genes (default: gene)\n";
  print STDERR " -h this help message\n";
  print STDERR "[VERSION]\n";
  print STDERR " 06-25-2012\n";
  print STDERR "[BUGS]\n";
  print STDERR " Please report bugs to salzamt\@bioinf.uni-leipzig.de\n";
  print STDERR "\n";
  exit(-1);
}

