#gero@bioinf.uni-leipzig.de
import sys, os, subprocess, argparse, time

parser = argparse.ArgumentParser(description='Pre-calculation for segemehl BAMs (in order to use DIEGO)')
parser.add_argument("-t", "--threads", dest='my_threads',  help="number of threads (default is one)")
parser.add_argument("-d", "--genome", dest='my_genome',required=True, help="specifies the path to the reference genome")
parser.add_argument("-a", "--anno", dest='my_anno',required=True, help="specifies the path to the annotation (bed 6 file of genebounderies with ENSG in column four and geneName in column five.)")
parser.add_argument("-l", "--list", dest='my_list', required=True, type=argparse.FileType('r'), help="sampleName tab-delimiter path/to/fileName")
parser.add_argument("-m", "--max_range", dest='my_max_range', type=int, help="specifies the maximal distance (default 7) to the exact splice site (due to majority clustering)")
parser.add_argument("-j", "--min_sup", dest='my_min_sup', type=int, help="specifies the minimum junction support (default 10))")
parser.add_argument("-o", "--outdir", dest='my_outdir', help="specifies the path to ouput directory (tmp files are also created in this directory) (default is '.')")

args = parser.parse_args()
genome=os.path.abspath(args.my_genome)
anno=os.path.abspath(args.my_anno)
if(args.my_threads):
    myThreads = int(args.my_threads)
else:
    myThreads = 1
if(args.my_max_range):
    maxRange = args.my_max_range
else:
    maxRange = 7
if(args.my_min_sup):
    min_sup = args.my_min_sup
else:
    min_sup = 10
  
if(args.my_outdir):
    outPath = os.path.abspath(args.my_outdir)
else:
    outPath=os.path.abspath(".")
os.chdir(outPath) 

#TODO here you have to specify the correct paths
realigner = "/path/to/testrealign.x"
pre_sub = "/path/to/pre_segemehl_sub.py"
haarz2yourtypeofjunctions = "/path/to/haarz2stats.py"

sampleH = {}

def get_jobs_parrallel(call_case):
    sleep_time = 1
    processes = list()
    for key, value in sampleH.iteritems():
        if call_case==1:
            call="samtools view -H "+value+"  > "+key+"_splits.sam"
        elif call_case==2:
            call="samtools view "+value+' | fgrep "XQ:i:" >> '+key+"_splits.sam"
        elif call_case==3:
            call="samtools view -b "+key+"_splits.sam | samtools sort - -T "+key+" -o "+key+"_splits.bam"
        elif call_case==4:
            call="samtools index "+key+"_splits.bam"
        elif call_case==5:
            call="rm "+key+"_splits.sam"
        elif call_case==6:
            call="python "+pre_sub+" -n normal_circ_junctions.bed -b "+key+"_splits.bam -m "+str(maxRange)+" 2> "+key+"_normal_circ.err > "+key+"_normal_circ.bed"
        processes.append(subprocess.Popen(call, shell=True))
        while len(processes) >= myThreads:
            time.sleep(sleep_time)
            processes = [proc for proc in processes if proc.poll() is None]
    for p in processes:
        if p.poll() is None:
            p.wait();
                
def build_datastructure():
    for line in args.my_list:
        columns = line.strip().split('\t')
        if columns[0] not in sampleH:
            sampleH[columns[0]]=columns[1]
        else:
            sys.stderr.write("error sample "+str(columns[0])+" is already present in data structure")
            sys.exit(1)
    

def runPipe():
    sys.stderr.write('Pre-calculation for segemehl BAMs (in order to use DoDAS) \n')
    sys.stderr.write('collecting all split reads from all samples\n')
    get_jobs_parrallel(1)
    get_jobs_parrallel(2)
    get_jobs_parrallel(3)
    get_jobs_parrallel(4)
    sys.stderr.write('merging\n')
    callString="samtools merge merged_split.bam"
    for key in sampleH:
        callString+=" "+str(key)+"_splits.bam"
    subprocess.call(callString,shell=True)
    subprocess.call("samtools view -h merged_split.bam 2> SAM.stderr | gzip -c > merged_split.sam.gz",shell=True)
    #subprocess.call("rm merged_split.bam",shell=True)
    sys.stderr.write('calling splice junctions with Haarz\n')
    subprocess.call(realigner+" -n -t "+str(myThreads)+" -d "+genome+" -q merged_split.sam.gz 2> realigner.stderr ",shell=True)
    sys.stderr.write('calculate junction support numbers for union \n')
    subprocess.call('python '+haarz2yourtypeofjunctions+" -u splicesites.bed -M -F -m "+str(min_sup)+" -C -N 2> normal_circ_junctions.stats > normal_circ_junctions.bed",shell=True)
    sys.stderr.write('calculate junction support numbers for individual samples \n')
    get_jobs_parrallel(6)

def buildMatrix():
    myMatrix = []
    myMatrix.append([])
    myMatrix[0].append("junction")
    myMatrix[0].append("type")
    firstSample=True
    for key in sampleH.iterkeys():
        myMatrix[0].append(key)
        myInput = open(key+"_normal_circ.bed",'r')
        lineNumber=0
        for line in myInput:
            columns = line.strip().split('\t')
            supports = columns[4]
            name_string=columns[0]+":"+columns[1]+"-"+columns[2]
            junc_type=columns[3]
            lineNumber+=1
            if firstSample:
                newListi=[]
                myMatrix.append(newListi)
                myMatrix[lineNumber].append(name_string)
                myMatrix[lineNumber].append(junc_type)
            myMatrix[lineNumber].append(supports)
        firstSample=False
    return myMatrix
        
def printMatrix(myMatrix):
    my_table = open("DoDAS_table.rm", "w")
    for zeilen in myMatrix:
        spaltencounter=0
        for spalten in zeilen:
            spaltencounter+=1
            if spaltencounter!=len(zeilen):
                my_table.write(spalten+"\t")
            else:
                my_table.write(spalten)
        my_table.write("\n")
    my_table.close()
   
def intersect_with_anno():
    subprocess.call('fgrep -v "junction" DoDAS_table.rm | awk \'{split($1,a,":");split(a[2],b,"-"); print a[1]"\t"b[1]"\t"b[2]"\t"$2}\' | sort -k1,1 -k2,2n > DoDAS_junctions.bed',shell=True)
    subprocess.call('intersectBed -a DoDAS_junctions.bed -b '+anno+' -f 1 -wa -wb | awk \'{print $1":"$2"-"$3"\t"$4"_w\t"$8"\t"$9}\' > DoDAS_overlap_w_all.rm',shell=True)
    subprocess.call('intersectBed -a DoDAS_junctions.bed -b '+anno+' -f 1 -wa | cut -f 1-3 | sort | uniq -c | awk \'{if($1 != 1) {print $2":"$3"-"$4}}\' > DoDAS_mulithits.rm',shell=True)
    subprocess.call('fgrep -v -f DoDAS_mulithits.rm DoDAS_overlap_w_all.rm > DoDAS_overlap_w.rm',shell=True)
    
    subprocess.call('awk \'{split ($1,a,":"); split(a[2],b,"-"); print a[1]"\t"b[1]"\t"b[2]}\' DoDAS_overlap_w_all.rm | fgrep -v -f - DoDAS_junctions.bed > DoDAS_overlap_o_all.bed',shell=True)
    subprocess.call('awk \'{print $1"\t"$2"\t"$2"\t"$1":"$2"-"$3" "$4}\' DoDAS_overlap_o_all.bed > DoDAS_leftis.bed',shell=True)
    subprocess.call('awk \'{print $1"\t"$3"\t"$3"\t"$1":"$2"-"$3" "$4}\' DoDAS_overlap_o_all.bed > DoDAS_righties.bed',shell=True)
    subprocess.call('intersectBed -a DoDAS_leftis.bed -b '+anno+' -wa | cut -f 4 | sort | uniq -c | awk \'{if($1 != 1) {print $2}}\' > DoDAS_leftis.multihits.rm',shell=True)
    subprocess.call('intersectBed -a DoDAS_righties.bed -b '+anno+' -wa | cut -f 4 | sort | uniq -c | awk \'{if($1 != 1) {print $2}}\' > DoDAS_righties.multihits.rm',shell=True)
    subprocess.call('intersectBed -a DoDAS_leftis.bed -b '+anno+' -wa -wb | awk -F "\t" \'{print $1"\t"$2"\t"$3"\t"$4" "$8" "$9}\' > DoDAS_leftis_hits.bed',shell=True)
    subprocess.call('intersectBed -a DoDAS_righties.bed -b '+anno+' -wa -wb | awk -F "\t" \'{print $1"\t"$2"\t"$3"\t"$4" "$8" "$9}\' > DoDAS_righties_hits.bed',shell=True)
    subprocess.call('fgrep -v -f DoDAS_leftis.multihits.rm DoDAS_leftis_hits.bed > DoDAS_leftis_clean.bed',shell=True)
    subprocess.call('fgrep -v -f DoDAS_righties.multihits.rm DoDAS_righties_hits.bed > DoDAS_righties_clean.bed',shell=True)
    subprocess.call('awk \'FNR==NR {a[$4]=1; next}{if ($4 in a){}else{print $4"\t"$5"_o\t"$6"\t"$7}}\' DoDAS_righties_hits.bed DoDAS_leftis_clean.bed > DoDAS_overlap_o.rm',shell=True) 
    subprocess.call('awk \'FNR==NR {a[$4]=1; next}{if ($4 in a){}else{print $4"\t"$5"_o\t"$6"\t"$7}}\' DoDAS_leftis_hits.bed DoDAS_righties_clean.bed >> DoDAS_overlap_o.rm',shell=True)
    subprocess.call('head -n 1  DoDAS_table.rm | awk \'{print $0 "\tgeneID\tgeneName"}\' > junction_table.txt',shell=True)
    subprocess.call('cat DoDAS_overlap_w.rm DoDAS_overlap_o.rm > DoDAS_overlap.rm',shell=True)
    subprocess.call('awk -F "\t" \'FNR==NR {a[$1]=$0; next} {if ($1 in a){ split(a[$1],b,"\t"); printf (b[1]"\t"b[2]); for(i=3; i<=NF; i++){printf "\t"$i}; print "\t"b[3]"\t"b[4]}}\' DoDAS_overlap.rm DoDAS_table.rm >> junction_table.txt',shell=True)
    subprocess.call('rm DoDAS_*.rm DoDAS_*.bed',shell=True)   
     
build_datastructure()     
runPipe()
printMatrix(buildMatrix())
intersect_with_anno()
sys.stderr.write('Now there should be a file named junction_table.txt in your specified out folder that can be used as input for DoDAS.py ')